div.transitions {
  display: inline-block;
  vertical-align:text-top;
  margin: 2em;
}

div.status {
  text-align: center;
  margin-top: 1em;
}
div.status table a {
  text-decoration: none;
}
div.status table {
  padding: 0px;
  border-spacing: 1px;
}
div.footer {
  margin-top: 2em;
  font-size: 60%;
}

.parameters {
  background: #eeeeee;
  padding: 0.3em;
  margin: 0.2em;
}

.parameters pre {
  font-size: 90%;
}

#footer {
  border-top: 2px solid #d70751;
  padding: 0.5em;
}

.level {
  text-align: left;
}

.src {
  background: #eeeeee;
}

.good {
  background: LightGreen;
  text-align: center;
}

.bad {
  background: Salmon;
  text-align: center;
}

.partial {
  background: #ffcc66;
  text-align: center;
}

.unknown {
  background: Cornsilk;
  text-align: center;
}

